@@ -52,7 +52,6 @@
           ]).
 :- use_module(library(apply)).
 :- use_module(library(error)).
-:- use_module(library(process)).
 :- use_module(library(option)).
 :- use_module(library(readutil)).
 :- use_module(library(lists)).
@@ -60,16 +59,18 @@
 :- use_module(library(xpath)).
 :- use_module(library(settings)).
 :- use_module(library(uri)).
+:- use_module(library(dcg/basics)).
 :- use_module(library(http/http_open)).
 :- use_module(library(http/json)).
 :- use_module(library(http/http_client), []).   % plugin for POST support
 :- use_module(library(prolog_config)).
 :- use_module(library(debug), [assertion/1]).
-:- use_module(library(pairs), [group_pairs_by_key/2, pairs_values/2]).
+:- use_module(library(pairs), [group_pairs_by_key/2]).
 % Stuff we may not have and may not need
 :- autoload(library(git)).
 :- autoload(library(sgml)).
 :- autoload(library(sha)).
+:- autoload(library(build/tools)).
 
 /** <module> A package manager for Prolog
 
@@ -621,15 +622,7 @@ url_menu_item(URL, URL=install_from(URL)).
 %     * url(+URL)
 %     Source for downloading the package
 %     * package_directory(+Dir)
-%     Directory into which to install the package.
-%     * global(+Boolean)
-%     If `true`, install in the XDG common application data path, making
-%     the pack accessible to everyone. If `false`, install in the XDG
-%     user application data path, making the pack accessible for the
-%     current user only.  If the option is absent, use the first
-%     existing and writable directory.  If that doesn't exist find
-%     locations where it can be created and prompt the user to do
-%     so.
+%     Directory into which to install the package
 %     * interactive(+Boolean)
 %     Use default answer without asking the user if there
 %     is a default action.
@@ -671,32 +664,19 @@ pack_install(Spec, Options) :-
 pack_install_dir(PackDir, Options) :-
     option(package_directory(PackDir), Options),
     !.
-pack_install_dir(PackDir, Options) :-
-    base_alias(Alias, Options),
-    absolute_file_name(Alias, PackDir,
+pack_install_dir(PackDir, _Options) :-          % TBD: global/user?
+    absolute_file_name(pack(.), PackDir,
                        [ file_type(directory),
                          access(write),
                          file_errors(fail)
                        ]),
     !.
-pack_install_dir(PackDir, Options) :-
+pack_install_dir(PackDir, Options) :-           % TBD: global/user?
     pack_create_install_dir(PackDir, Options).
 
-base_alias(Alias, Options) :-
-    option(global(true), Options),
-    !,
-    Alias = common_app_data(pack).
-base_alias(Alias, Options) :-
-    option(global(false), Options),
-    !,
-    Alias = user_app_data(pack).
-base_alias(Alias, _Options) :-
-    Alias = pack('.').
-
 pack_create_install_dir(PackDir, Options) :-
-    base_alias(Alias, Options),
     findall(Candidate = create_dir(Candidate),
-            ( absolute_file_name(Alias, Candidate, [solutions(all)]),
+            ( absolute_file_name(pack(.), Candidate, [solutions(all)]),
               \+ exists_file(Candidate),
               \+ exists_directory(Candidate),
               file_directory_name(Candidate, Super),
@@ -1106,19 +1086,10 @@ pack_post_install(Pack, PackDir, Options) :-
 pack_rebuild(Pack) :-
     current_pack(Pack, PackDir),
     !,
-    pack_clean(Pack, PackDir, distclean),
-    post_install_foreign(Pack, PackDir, []).
+    post_install_foreign(Pack, PackDir, [rebuild(true)]).
 pack_rebuild(Pack) :-
     existence_error(pack, Pack).
 
-pack_clean(Pack, PackDir, How) :-
-    (   var(PackDir)
-    ->  current_pack(Pack, PackDir)
-    ;   true
-    ),
-    catch(pack_make(PackDir, [How], []), E,
-          print_message(warning, E)).
-
 %!  pack_rebuild is det.
 %
 %   Rebuild foreign components of all packages.
@@ -1135,22 +1106,26 @@ pack_rebuild :-
 %   Install foreign parts of the package.
 
 post_install_foreign(Pack, PackDir, Options) :-
-    is_foreign_pack(PackDir),
+    is_foreign_pack(PackDir, _),
     !,
     option(rebuild(Rebuild), Options, if_absent),
     (   Rebuild == if_absent,
         foreign_present(PackDir)
     ->  print_message(informational, pack(kept_foreign(Pack)))
-    ;   (   Rebuild == true
-        ->  pack_clean(Pack, PackDir, distclean)
-        ;   true
+    ;   BuildSteps0 = [[dependencies], [configure], build, [test], install],
+        (   Rebuild == true
+        ->  BuildSteps = [distclean|BuildSteps0]
+        ;   BuildSteps = BuildSteps0
         ),
-        setup_path,
-        save_build_environment(PackDir),
-        configure_foreign(PackDir, Options),
-        make_foreign(PackDir, Options)
+        build_steps(BuildSteps, PackDir, Options)
     ).
-post_install_foreign(_, _, _).
+
+
+%!  foreign_present(+PackDir) is semidet.
+%
+%   True if we find one or more modules  in the pack `lib` directory for
+%   the current architecture. Does not check   that these can be loaded,
+%   nor whether all required modules are present.
 
 foreign_present(PackDir) :-
     current_prolog_flag(arch, Arch),
@@ -1164,399 +1139,25 @@ foreign_present(PackDir) :-
     expand_file_name(Pattern, Files),
     Files \== [].
 
-is_foreign_pack(PackDir) :-
-    foreign_file(File),
+%!  is_foreign_pack(+PackDir, -Type) is nondet.
+%
+%   True when PackDir contains  files  that   indicate  the  need  for a
+%   specific class of build tools indicated by Type.
+
+is_foreign_pack(PackDir, Type) :-
+    foreign_file(File, Type),
     directory_file_path(PackDir, File, Path),
-    exists_file(Path),
-    !.
+    exists_file(Path).
 
-foreign_file('configure.in').
-foreign_file('configure.ac').
-foreign_file('configure').
-foreign_file('Makefile').
-foreign_file('makefile').
-foreign_file('CMakeLists.txt').
-
-
-%!  configure_foreign(+PackDir, +Options) is det.
-%
-%   Run configure if it exists.  If =|configure.ac|= or =|configure.in|=
-%   exists, first run =autoheader= and =autoconf=
-
-configure_foreign(PackDir, Options) :-
-    directory_file_path(PackDir, 'CMakeLists.txt', CMakeFile),
-    exists_file(CMakeFile),
-    !,
-    cmake_configure_foreign(PackDir, Options).
-configure_foreign(PackDir, Options) :-
-    make_configure(PackDir, Options),
-    directory_file_path(PackDir, configure, Configure),
-    exists_file(Configure),
-    !,
-    build_environment(BuildEnv),
-    findall(Opt, configure_option(Opt), Opts),
-    run_process(path(bash), [Configure|Opts],
-                [ env(BuildEnv),
-                  directory(PackDir)
-                ]).
-configure_foreign(_, _).
-
-configure_option(Opt) :-
-    prolog_prefix(Prefix),
-    format(atom(Opt), '--prefix=~w', [Prefix]).
-
-make_configure(PackDir, _Options) :-
-    directory_file_path(PackDir, 'configure', Configure),
-    exists_file(Configure),
-    !.
-make_configure(PackDir, _Options) :-
-    autoconf_master(ConfigMaster),
-    directory_file_path(PackDir, ConfigMaster, ConfigureIn),
-    exists_file(ConfigureIn),
-    !,
-    run_process(path(autoheader), [], [directory(PackDir)]),
-    run_process(path(autoconf),   [], [directory(PackDir)]).
-make_configure(_, _).
-
-autoconf_master('configure.ac').
-autoconf_master('configure.in').
-
-%!  cmake_configure_foreign(+PackDir, +Options) is det.
-%
-%   Create a `build` directory in PackDir and run `cmake [options] ..`
-
-cmake_configure_foreign(PackDir, _Options) :-
-    directory_file_path(PackDir, build, BuildDir),
-    make_directory_path(BuildDir),
-    findall(Opt, cmake_option(Opt), Argv, [..]),
-    build_environment(BuildEnv),
-    run_process(path(cmake), Argv,
-                [directory(BuildDir),
-                 env(BuildEnv)]).
-
-cmake_option(CDEF) :-
-    current_prolog_flag(executable, Exe),
-    format(atom(CDEF), '-DSWIPL=~w', [Exe]).
-cmake_option(CDEF) :-
-    prolog_prefix(Prefix),
-    format(atom(CDEF), '-DCMAKE_INSTALL_PREFIX=~w', [Prefix]).
-cmake_option(Opt) :-
-    has_program(path(ninja), _),
-    member(Opt, ['-G', 'Ninja']).
-
-%!  make_foreign(+PackDir, +Options) is det.
-%
-%   Generate the foreign executable.
-
-make_foreign(PackDir, Options) :-
-    pack_make(PackDir, [all, check, install], Options).
-
-pack_make(PackDir, Targets, _Options) :-
-    directory_file_path(PackDir, 'CMakeLists.txt', CMakefile),
-    exists_file(CMakefile),
-    directory_file_path(PackDir, 'build', BuildDir),
-    exists_directory(BuildDir),
-    !,
-    (   Targets == [distclean]
-    ->  delete_directory_contents(BuildDir)
-    ;   build_environment(BuildEnv),
-        ProcessOptions = [ directory(BuildDir), env(BuildEnv) ],
-        forall(member(Target, Targets),
-               run_cmake_target(Target, BuildDir, ProcessOptions))
-    ).
-pack_make(PackDir, Targets, _Options) :-
-    directory_file_path(PackDir, 'Makefile', Makefile),
-    exists_file(Makefile),
-    !,
-    build_environment(BuildEnv),
-    ProcessOptions = [ directory(PackDir), env(BuildEnv) ],
-    forall(member(Target, Targets),
-           run_process(path(make), [Target], ProcessOptions)).
-pack_make(_, _, _).
-
-%!  run_cmake_target(+Target, +BuildDir, +ProcessOptions) is det.
-%
-%   Run a process step for CMake. We handle `check` using `ctest`, `all`
-%   using ``cmake --build .`` and `install` using ``cmake --install .``.
-
-run_cmake_target(check, BuildDir, ProcessOptions) :-
-    !,
-    (   directory_file_path(BuildDir, 'CTestTestfile.cmake', TestFile),
-        exists_file(TestFile)
-    ->  run_process(path(ctest), [], ProcessOptions)
-    ;   true
-    ).
-run_cmake_target(all, _BuildDir, ProcessOptions) :-
-    !,
-    run_process(path(cmake), ['--build', '.'], ProcessOptions).
-run_cmake_target(install, _BuildDir, ProcessOptions) :-
-    !,
-    run_process(path(cmake), ['--install', '.'], ProcessOptions).
-run_cmake_target(Target, BuildDir, ProcessOptions) :-
-    directory_file_path(BuildDir, 'build.ninja', NinjaFile),
-    exists_file(NinjaFile),
-    !,
-    run_process(path(ninja), [Target], ProcessOptions).
-run_cmake_target(Target, _, ProcessOptions) :-
-    run_process(path(make), [Target], ProcessOptions).
-
-%!  save_build_environment(+PackDir)
-%
-%   Create  a  shell-script  build.env  that    contains  the  build
-%   environment.
-
-save_build_environment(PackDir) :-
-    directory_file_path(PackDir, 'buildenv.sh', EnvFile),
-    build_environment(Env),
-    setup_call_cleanup(
-        open(EnvFile, write, Out),
-        write_env_script(Out, Env),
-        close(Out)).
-
-write_env_script(Out, Env) :-
-    format(Out,
-           '# This file contains the environment that can be used to\n\c
-                # build the foreign pack outside Prolog.  This file must\n\c
-                # be loaded into a bourne-compatible shell using\n\c
-                #\n\c
-                #   $ source buildenv.sh\n\n',
-           []),
-    forall(member(Var=Value, Env),
-           format(Out, '~w=\'~w\'\n', [Var, Value])),
-    format(Out, '\nexport ', []),
-    forall(member(Var=_, Env),
-           format(Out, ' ~w', [Var])),
-    format(Out, '\n', []).
-
-build_environment(Env) :-
-    findall(Name=Value, environment(Name, Value), UserEnv),
-    findall(Name=Value,
-            ( def_environment(Name, Value),
-              \+ memberchk(Name=_, UserEnv)
-            ),
-            DefEnv),
-    append(UserEnv, DefEnv, Env).
-
-
-%!  environment(-Name, -Value) is nondet.
-%
-%   Hook  to  define  the  environment   for  building  packs.  This
-%   Multifile hook extends the  process   environment  for  building
-%   foreign extensions. A value  provided   by  this  hook overrules
-%   defaults provided by def_environment/2. In  addition to changing
-%   the environment, this may be used   to pass additional values to
-%   the environment, as in:
-%
-%     ==
-%     prolog_pack:environment('USER', User) :-
-%         getenv('USER', User).
-%     ==
-%
-%   @param Name is an atom denoting a valid variable name
-%   @param Value is either an atom or number representing the
-%          value of the variable.
-
-
-%!  def_environment(-Name, -Value) is nondet.
-%
-%   True if Name=Value must appear in   the environment for building
-%   foreign extensions.
-
-def_environment('PATH', Value) :-
-    getenv('PATH', PATH),
-    current_prolog_flag(executable, Exe),
-    file_directory_name(Exe, ExeDir),
-    prolog_to_os_filename(ExeDir, OsExeDir),
-    (   current_prolog_flag(windows, true)
-    ->  Sep = (;)
-    ;   Sep = (:)
-    ),
-    atomic_list_concat([OsExeDir, Sep, PATH], Value).
-def_environment('SWIPL', Value) :-
-    current_prolog_flag(executable, Value).
-def_environment('SWIPLVERSION', Value) :-
-    current_prolog_flag(version, Value).
-def_environment('SWIHOME', Value) :-
-    current_prolog_flag(home, Value).
-def_environment('SWIARCH', Value) :-
-    current_prolog_flag(arch, Value).
-def_environment('PACKSODIR', Value) :-
-    current_prolog_flag(arch, Arch),
-    atom_concat('lib/', Arch, Value).
-def_environment('SWISOLIB', Value) :-
-    current_prolog_flag(c_libplso, Value).
-def_environment('SWILIB', '-lswipl').
-def_environment('CC', Value) :-
-    (   getenv('CC', Value)
-    ->  true
-    ;   default_c_compiler(Value)
-    ->  true
-    ;   current_prolog_flag(c_cc, Value)
-    ).
-def_environment('LD', Value) :-
-    (   getenv('LD', Value)
-    ->  true
-    ;   current_prolog_flag(c_cc, Value)
-    ).
-def_environment('CFLAGS', Value) :-
-    (   getenv('CFLAGS', SystemFlags)
-    ->  Extra = [' ', SystemFlags]
-    ;   Extra = []
-    ),
-    current_prolog_flag(c_cflags, Value0),
-    current_prolog_flag(home, Home),
-    atomic_list_concat([Value0, ' -I"', Home, '/include"' | Extra], Value).
-def_environment('LDSOFLAGS', Value) :-
-    (   getenv('LDFLAGS', SystemFlags)
-    ->  Extra = [SystemFlags|System]
-    ;   Extra = System
-    ),
-    (   current_prolog_flag(windows, true)
-    ->  current_prolog_flag(home, Home),
-        atomic_list_concat(['-L"', Home, '/bin"'], SystemLib),
-        System = [SystemLib]
-    ;   apple_bundle_libdir(LibDir)
-    ->  atomic_list_concat(['-L"', LibDir, '"'], SystemLib),
-        System = [SystemLib]
-    ;   current_prolog_flag(c_libplso, '')
-    ->  System = []                 % ELF systems do not need this
-    ;   prolog_library_dir(SystemLibDir),
-        atomic_list_concat(['-L"',SystemLibDir,'"'], SystemLib),
-        System = [SystemLib]
-    ),
-    current_prolog_flag(c_ldflags, LDFlags),
-    atomic_list_concat([LDFlags, '-shared' | Extra], ' ', Value).
-def_environment('SOEXT', Value) :-
-    current_prolog_flag(shared_object_extension, Value).
-def_environment(Pass, Value) :-
-    pass_env(Pass),
-    getenv(Pass, Value).
-
-pass_env('TMP').
-pass_env('TEMP').
-pass_env('USER').
-pass_env('HOME').
-
-:- multifile
-    prolog:runtime_config/2.
-
-prolog_library_dir(Dir) :-
-    prolog:runtime_config(c_libdir, Dir),
-    !.
-prolog_library_dir(Dir) :-
-    current_prolog_flag(home, Home),
-    (   current_prolog_flag(c_libdir, Rel)
-    ->  atomic_list_concat([Home, Rel], /, Dir)
-    ;   current_prolog_flag(arch, Arch)
-    ->  atomic_list_concat([Home, lib, Arch], /, Dir)
-    ).
-
-%!  default_c_compiler(-CC) is semidet.
-%
-%   Try to find a  suitable  C   compiler  for  compiling  packages with
-%   foreign code.
-%
-%   @tbd Needs proper defaults for Windows.  Find MinGW?  Find MSVC?
-
-default_c_compiler(CC) :-
-    preferred_c_compiler(CC),
-    has_program(path(CC), _),
-    !.
-
-preferred_c_compiler(gcc).
-preferred_c_compiler(clang).
-preferred_c_compiler(cc).
-
-
-                 /*******************************
-                 *             PATHS            *
-                 *******************************/
-
-setup_path :-
-    has_program(path(make), _),
-    has_program(path(gcc), _),
-    !.
-setup_path :-
-    current_prolog_flag(windows, true),
-    !,
-    (   mingw_extend_path
-    ->  true
-    ;   print_message(error, pack(no_mingw))
-    ).
-setup_path.
-
-has_program(Program, Path) :-
-    exe_options(ExeOptions),
-    absolute_file_name(Program, Path,
-                       [ file_errors(fail)
-                       | ExeOptions
-                       ]).
-
-exe_options(Options) :-
-    current_prolog_flag(windows, true),
-    !,
-    Options = [ extensions(['',exe,com]), access(read) ].
-exe_options(Options) :-
-    Options = [ access(execute) ].
-
-mingw_extend_path :-
-    mingw_root(MinGW),
-    directory_file_path(MinGW, bin, MinGWBinDir),
-    atom_concat(MinGW, '/msys/*/bin', Pattern),
-    expand_file_name(Pattern, MsysDirs),
-    last(MsysDirs, MSysBinDir),
-    prolog_to_os_filename(MinGWBinDir, WinDirMinGW),
-    prolog_to_os_filename(MSysBinDir, WinDirMSYS),
-    getenv('PATH', Path0),
-    atomic_list_concat([WinDirMSYS, WinDirMinGW, Path0], ';', Path),
-    setenv('PATH', Path).
-
-mingw_root(MinGwRoot) :-
-    current_prolog_flag(executable, Exe),
-    sub_atom(Exe, 1, _, _, :),
-    sub_atom(Exe, 0, 1, _, PlDrive),
-    Drives = [PlDrive,c,d],
-    member(Drive, Drives),
-    format(atom(MinGwRoot), '~a:/MinGW', [Drive]),
-    exists_directory(MinGwRoot),
-    !.
-
-%!  prolog_prefix(-Prefix) is semidet.
-%
-%   Return the directory that can be  passed into `configure` or `cmake`
-%   to install executables and other  related   resources  in  a similar
-%   location as SWI-Prolog itself.  Tries these rules:
-%
-%     1. If the Prolog flag `pack_prefix` at a writable directory, use
-%        this.
-%     2. If the current executable can be found on $PATH and the parent
-%        of the directory of the executable is writable, use this.
-%     3. If the user has a writable ``~/bin`` directory, use ``~``.
-
-prolog_prefix(Prefix) :-
-    current_prolog_flag(pack_prefix, Prefix),
-    access_file(Prefix, write),
-    !.
-prolog_prefix(Prefix) :-
-    current_prolog_flag(os_argv, [Name|_]),
-    has_program(path(Name), EXE),
-    file_directory_name(EXE, Bin),
-    file_directory_name(Bin, Prefix0),
-    (   local_prefix(Prefix0, Prefix1)
-    ->  Prefix = Prefix1
-    ;   Prefix = Prefix0
-    ),
-    access_file(Prefix, write),
-    !.
-prolog_prefix(Prefix) :-
-    expand_file_name(~, [UserHome]),
-    directory_file_path(UserHome, bin, BinDir),
-    exists_directory(BinDir),
-    access_file(BinDir, write),
-    !,
-    Prefix = UserHome.
-
-local_prefix('/usr', '/usr/local').
+foreign_file('CMakeLists.txt', cmake).
+foreign_file('configure',      configure).
+foreign_file('configure.in',   autoconf).
+foreign_file('configure.ac',   autoconf).
+foreign_file('Makefile.am',    automake).
+foreign_file('Makefile',       make).
+foreign_file('makefile',       make).
+foreign_file('conanfile.txt',  conan).
+foreign_file('conanfile.py',   conan).
 
 
                  /*******************************
@@ -1871,12 +1472,6 @@ version([H|T]) -->
     ;   { T = [] }
     ).
 
-integer(H)    --> digit(D0), digits(L), { number_codes(H, [D0|L]) }.
-digit(D)      --> [D], { code_type(D, digit) }.
-digits([H|T]) --> digit(H), !, digits(T).
-digits([])    --> [].
-
-
                  /*******************************
                  *       QUERY CENTRAL DB       *
                  *******************************/
@@ -2332,129 +1927,6 @@ version_pack(pack(VersionAtom,URLs,SubDeps),
     atom_version(VersionAtom, Version).
 
 
-                 /*******************************
-                 *          RUN PROCESSES       *
-                 *******************************/
-
-%!  run_process(+Executable, +Argv, +Options) is det.
-%
-%   Run Executable.  Defined options:
-%
-%     * directory(+Dir)
-%     Execute in the given directory
-%     * output(-Out)
-%     Unify Out with a list of codes representing stdout of the
-%     command.  Otherwise the output is handed to print_message/2
-%     with level =informational=.
-%     * error(-Error)
-%     As output(Out), but messages are printed at level =error=.
-%     * env(+Environment)
-%     Environment passed to the new process.
-
-run_process(Executable, Argv, Options) :-
-    \+ option(output(_), Options),
-    \+ option(error(_), Options),
-    current_prolog_flag(unix, true),
-    current_prolog_flag(threads, true),
-    !,
-    process_create_options(Options, Extra),
-    process_create(Executable, Argv,
-                   [ stdout(pipe(Out)),
-                     stderr(pipe(Error)),
-                     process(PID)
-                   | Extra
-                   ]),
-    thread_create(relay_output([output-Out, error-Error]), Id, []),
-    process_wait(PID, Status),
-    thread_join(Id, _),
-    (   Status == exit(0)
-    ->  true
-    ;   throw(error(process_error(process(Executable, Argv), Status), _))
-    ).
-run_process(Executable, Argv, Options) :-
-    process_create_options(Options, Extra),
-    setup_call_cleanup(
-        process_create(Executable, Argv,
-                       [ stdout(pipe(Out)),
-                         stderr(pipe(Error)),
-                         process(PID)
-                       | Extra
-                       ]),
-        (   read_stream_to_codes(Out, OutCodes, []),
-            read_stream_to_codes(Error, ErrorCodes, []),
-            process_wait(PID, Status)
-        ),
-        (   close(Out),
-            close(Error)
-        )),
-    print_error(ErrorCodes, Options),
-    print_output(OutCodes, Options),
-    (   Status == exit(0)
-    ->  true
-    ;   throw(error(process_error(process(Executable, Argv), Status), _))
-    ).
-
-process_create_options(Options, Extra) :-
-    option(directory(Dir), Options, .),
-    (   option(env(Env), Options)
-    ->  Extra = [cwd(Dir), env(Env)]
-    ;   Extra = [cwd(Dir)]
-    ).
-
-relay_output([]) :- !.
-relay_output(Output) :-
-    pairs_values(Output, Streams),
-    wait_for_input(Streams, Ready, infinite),
-    relay(Ready, Output, NewOutputs),
-    relay_output(NewOutputs).
-
-relay([], Outputs, Outputs).
-relay([H|T], Outputs0, Outputs) :-
-    selectchk(Type-H, Outputs0, Outputs1),
-    (   at_end_of_stream(H)
-    ->  close(H),
-        relay(T, Outputs1, Outputs)
-    ;   read_pending_codes(H, Codes, []),
-        relay(Type, Codes),
-        relay(T, Outputs0, Outputs)
-    ).
-
-relay(error,  Codes) :-
-    set_prolog_flag(message_context, []),
-    print_error(Codes, []).
-relay(output, Codes) :-
-    print_output(Codes, []).
-
-print_output(OutCodes, Options) :-
-    option(output(Codes), Options),
-    !,
-    Codes = OutCodes.
-print_output(OutCodes, _) :-
-    print_message(informational, pack(process_output(OutCodes))).
-
-print_error(OutCodes, Options) :-
-    option(error(Codes), Options),
-    !,
-    Codes = OutCodes.
-print_error(OutCodes, _) :-
-    phrase(classify_message(Level), OutCodes, _),
-    print_message(Level, pack(process_output(OutCodes))).
-
-classify_message(error) -->
-    string(_), "fatal:",
-    !.
-classify_message(error) -->
-    string(_), "error:",
-    !.
-classify_message(warning) -->
-    string(_), "warning:",
-    !.
-classify_message(informational) -->
-    [].
-
-string([]) --> [].
-string([H|T]) --> [H], string(T).
-
 
 %!  pack_attach(+Dir, +Options) is det.
 %
@@ -2702,9 +2174,6 @@ message(pack(no_upgrade_info(Pack))) -->
 candidate_dirs([]) --> [].
 candidate_dirs([H|T]) --> [ nl, '    ~w'-[H] ], candidate_dirs(T).
 
-message(no_mingw) -->
-    [ 'Cannot find MinGW and/or MSYS.'-[] ].
-
                                                 % Questions
 message(resolve_remove) -->
     [ nl, 'Please select an action:', nl, nl ].
