@@ -47,7 +47,7 @@ test_tmp_module :-
 
 :- begin_tests(tmp_module).
 
-test(nqueens) :-
+test(nqueens, condition(current_prolog_flag(threads,true))) :-
 	test_tmp_module(100).
 test(current_op, condition(current_prolog_flag(threads,true))) :-
 	forall(between(1, 100, _),
@@ -72,8 +72,10 @@ queens_file(File) :-
 	atomic_list_concat([Dir, /, 'data/queens.pl'], File).
 
 tmp_queens(S) :-
-	N is random(1<<62),		% was uuid(UUID), but that is a package
-	format(atom(UUID), 'tmp-~d', [N]),
+	N is random(1<<62), % was uuid(UUID), but that is a package
+	thread_self(Me),
+	thread_property(Me, id(Id)),
+	format(atom(UUID), 'tmp-~w-~d', [Id, N]),
 	queens_file(Queens),
 	in_temporary_module(
 	    UUID,
