@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam,
+    Copyright (c)  1985-2021, University of Amsterdam,
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -2103,6 +2104,37 @@ typedef struct
 #define SIG_PLABORT	  (SIG_PROLOG_OFFSET+4)
 #define SIG_TUNE_GC	  (SIG_PROLOG_OFFSET+5)
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Guard against C-stack overflows. This is   done  for POSIX systems using
+the  alternative  signal  stack.  The  signal    handling   is  done  by
+alt_segv_handler().
+
+Note that we  use  setjmp()  rather   than  sigsetjmp().  The  latter is
+simpler, but a lot slower as it  implies   a  system  call. We assume no
+other signals are involved and unblock SIGSEGV by hand.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+#if defined(HAVE_SIGALTSTACK) && !defined(__SANITIZE_ADDRESS__)
+#define O_C_STACK_GUARDED 1
+#define C_STACK_OVERFLOW_GUARDED(rc, code, cleanup) \
+	do						\
+	{ LD->signal.sig_critical = TRUE;		\
+	  if ( setjmp(LD->signal.context) )		\
+	  { cleanup;					\
+	    unblockSignal(SIGSEGV);			\
+	    rc = PL_resource_error("c_stack");		\
+	  } else					\
+	  { rc = code;					\
+	  }						\
+	  LD->signal.sig_critical = FALSE;		\
+	} while(0)
+#else
+#define C_STACK_OVERFLOW_GUARDED(rc, code, cleanup) \
+	do						\
+	{ rc = code;						\
+	} while(0)
+#endif
+
 
 		 /*******************************
 		 *	       COMPARE		*
